@@ -992,3 +992,44 @@ def test_fam_array_get_any_d():
 
     post = fam.get_any(np.array(["2022-01", "2023-01", "1988-01"], np.datetime64))
     assert post == []
+
+
+def test_fam_get_dt64_a():
+    a1 = np.array(("2023", "1854", "1988"), np.datetime64)
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+
+    k1 = np.datetime64("1988-01-01")
+    with pytest.raises(KeyError):
+        _ = fam[k1]
+
+    k2 = np.datetime64("2023-01-01")
+    with pytest.raises(KeyError):
+        _ = fam[k2]
+
+def test_fam_get_dt64_b():
+    a1 = np.array(("2023", "1854", "1988"), np.datetime64)
+    fam = FrozenAutoMap(list(a1))
+
+    k1 = np.datetime64("1988-01-01")
+    with pytest.raises(KeyError):
+        _ = fam[k1]
+
+    k2 = np.datetime64("2023-01-01")
+    with pytest.raises(KeyError):
+        _ = fam[k2]
+
+
+def test_am_get_dt64_a():
+    a1 = np.array(("2023", "1854", "1988"), np.datetime64)
+    a1.flags.writeable = False
+    fam = AutoMap(a1)
+
+    k1 = np.datetime64("1988-01-01")
+    with pytest.raises(KeyError):
+        _ = fam[k1]
+
+    k2 = np.datetime64("2023-01-01")
+    with pytest.raises(KeyError):
+        _ = fam[k2]
+
